#include <iostream>
#include <fstream>
#include <vector>
#include <libplayerc++/playerc++.h>
#include "args.h"
using namespace std;
using namespace PlayerCc;

void getOtherRobotPoses(SimulationProxy &simp) {
 double x,y,a,t,t0; //robot position
  for(int i=0; i<teamSize; i++) {
    if(i!=id) {
      char rname[256];
      sprintf(rname,"robot%d",i);
      simp.GetPose2d (rname, x, y, a);
      cerr<<"Robot "<<i<<" Pose (x,y,theta): "<<x<<","<<y<<","<<a<<endl;
    }
  }
}

int main(int argc, char *argv[]) {
// Parse command line options
  parse_args(argc,argv);

  cout<<"Robot "<<id<<" of "<<teamSize<<endl;
  cout<<"Destination: "<<destX<<","<<destY<<endl;

  PlayerClient    stage(gHostname,gPort);
  SimulationProxy simp(&stage,0); //used to get real position of the robot



  PlayerClient    robot(gHostname,gPort+id);
  SonarProxy      sp(&robot,0);
  Position2dProxy pp(&robot,0);

  double x,y,a,t,t0; //robot position

  
  robot.Read();

  getOtherRobotPoses(simp);


  t0=sp.GetDataTime();//starting time;
  cerr<<"t0="<<t0<<endl;
  for(;;) {
    double turnrate, speed;

    // read from the proxies
    robot.Read();

    x=pp.GetXPos();
    y=pp.GetYPos();
    a=pp.GetYaw(); 
    t=sp.GetDataTime();//current simulation time in seconds;
    cerr<<"Elapsed Time: "<<t-t0<<endl; 
    cerr<<"Robot Pose (x,y,theta): "<<x<<","<<y<<","<<a<<endl;
    getOtherRobotPoses(simp);
    cerr<<"Sonar Data: ";
    for(unsigned int i=0; i<sp.GetCount(); i++){
      cerr<<sp[i]<<" ";
    }
    cerr<<endl;
    //you can get the post of the sonars in robot's local coordinate system using:
    player_pose3d_t  pose;
    sp.RequestGeom();
    //cerr<<"Sonar pose (x,y,theta) in local coordinates:";
    //for(unsigned int i=0; i<sp.GetPoseCount(); i++){
    //  pose=sp.GetPose(i);
    //   cerr<<"("<<pose.px<<","<<pose.py<<","<<pose.pyaw<<") ";
    //}
    //cerr<<endl;

    // Treat the position of the other robots as special obstacles 
    // (take into account the radius of the robot)
    // Use the sonar (less than max range) and the other-robot-pose to 
    // create repulsive forces
    // Use the destination destX,destY to create attractive forces.
    // Use the vector sum of the force to change speed and turnrate 

    // do simple collision avoidance
    if((sp[0] + sp[1]) < (sp[6] + sp[7]))
      turnrate = dtor(-20); // turn 20 degrees per second
    else
      turnrate = dtor(20);

    if(sp[3] < 0.500)
      speed = 0.0;
    else
      speed = 0.200;
        
    // command the motors
    pp.SetSpeed(speed, turnrate);      
  }
}

